@@ -30,7 +30,7 @@
 @file      absmatcher.h
 @brief     RE/flex abstract matcher base class and pattern matcher class
 @author    Robert van Engelen - engelen@genivia.com
-@copyright (c) 2016-2020, Robert van Engelen, Genivia Inc. All rights reserved.
+@copyright (c) 2016-2022, Robert van Engelen, Genivia Inc. All rights reserved.
 @copyright (c) BSD-3 License - see LICENSE.txt
 */
 
@@ -43,7 +43,7 @@
 /// This compile-time option speeds up matching, but slows input().
 #define WITH_FAST_GET
 
-/// This compile-time option adds span(), line(), wline(), speeds up buffer shifting and lineno().
+/// This compile-time option adds span(), line(), wline(), bol(), eol()
 #define WITH_SPAN
 
 #include <reflex/convert.h>
@@ -66,7 +66,7 @@ inline int isword(int c) ///< Character to check
 
 /// The abstract matcher base class template defines an interface for all pattern matcher engines.
 /**
-The buffer expands when matches do not fit.  The buffer size is initially 2*BLOCK size.
+The buffer expands when matches do not fit.  The buffer size is initially BUFSZ.
 
 ```
       _________________
@@ -102,7 +102,17 @@ class AbstractMatcher {
     static const int UNK      = 256;        ///< unknown/undefined character meta-char marker
     static const int BOB      = 257;        ///< begin of buffer meta-char marker
     static const int EOB      = EOF;        ///< end of buffer meta-char marker
-    static const size_t BLOCK = (256*1024); ///< buffer size and growth, buffer is initially 2*BLOCK size, at least 4096 bytes
+#ifndef REFLEX_BUFSZ
+    static const size_t BUFSZ = (64*1024);  ///< initial buffer size, at least 4096 bytes
+#else
+    static const size_t BUFSZ = REFLEX_BUFSZ;
+#endif
+#ifndef REFLEX_BOLSZ
+    static const size_t BOLSZ = (256*1024); ///< max begin of line size till match to retain in memory by growing the buffer
+#else
+    static const size_t BOLSZ = REFLEX_BOLSZ;
+#endif
+    static const size_t BLOCK = 4096;       ///< minimum remaining unused space in the buffer, to prevent excessive shifting
     static const size_t REDO  = 0x7FFFFFFF; ///< reflex::Matcher::accept() returns "redo" with reflex::Matcher option "A"
     static const size_t EMPTY = 0xFFFFFFFF; ///< accept() returns "empty" last split at end of input
   };
@@ -348,7 +358,7 @@ class AbstractMatcher {
     }
     if (!own_)
     {
-      max_ = 2 * Const::BLOCK;
+      max_ = Const::BUFSZ;
 #if defined(WITH_REALLOC)
 #if (defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(_WIN64) || defined(__BORLANDC__)) && !defined(__CYGWIN__)
       buf_ = static_cast<char*>(_aligned_malloc(max_, 4096));
@@ -380,9 +390,10 @@ class AbstractMatcher {
 #endif
     lpb_ = buf_;
     lno_ = 1;
-#if !defined(WITH_SPAN)
-    cno_ = 0;
+#if defined(WITH_SPAN)
+    cpb_ = buf_;
 #endif
+    cno_ = 0;
     num_ = 0;
     own_ = true;
     eof_ = false;
@@ -513,9 +524,10 @@ class AbstractMatcher {
 #endif
       lpb_ = buf_;
       lno_ = 1;
-#if !defined(WITH_SPAN)
-      cno_ = 0;
+#if defined(WITH_SPAN)
+      cpb_ = buf_;
 #endif
+      cno_ = 0;
       num_ = 0;
       own_ = false;
       eof_ = true;
@@ -615,34 +627,19 @@ class AbstractMatcher {
 #if defined(WITH_SPAN)
     if (lpb_ < txt_)
     {
-      const char *s = bol_;
+      const char *s = lpb_;
       const char *t = txt_;
-      // clang/gcc 4-way vectorizable loop
-      while (t - 4 >= s)
-      {
-        if ((t[-1] == '\n') | (t[-2] == '\n') | (t[-3] == '\n') | (t[-4] == '\n'))
-          break;
-        t -= 4;
-      }
-      // epilogue
-      if (--t >= s && *t != '\n')
-        if (--t >= s && *t != '\n')
-          if (--t >= s && *t != '\n')
-            --t;
-      bol_ = const_cast<char*>(t + 1);
-      lpb_ = txt_;
-      size_t n = lno_;
-
+      size_t n = 0;
 #if defined(HAVE_AVX512BW) && (!defined(_MSC_VER) || defined(_WIN64))
-      if (s + 63 > t && have_HW_AVX512BW())
+      if (have_HW_AVX512BW())
       {
         n += simd_nlcount_avx512bw(s, t);
       }
-      else if (s + 31 > t && have_HW_AVX2())
+      else if (have_HW_AVX2())
       {
         n += simd_nlcount_avx2(s, t);
       }
-      else if (have_HW_SSE2())
+      else
       {
         __m128i vlcn = _mm_set1_epi8('\n');
         while (s + 15 <= t)
@@ -655,11 +652,11 @@ class AbstractMatcher {
         }
       }
 #elif defined(HAVE_AVX2)
-      if (s + 31 > t && have_HW_AVX2())
+      if (have_HW_AVX2())
       {
         n += simd_nlcount_avx2(s, t);
       }
-      else if (have_HW_SSE2())
+      else
       {
         __m128i vlcn = _mm_set1_epi8('\n');
         while (s + 15 <= t)
@@ -672,17 +669,14 @@ class AbstractMatcher {
         }
       }
 #elif defined(HAVE_SSE2)
-      if (have_HW_SSE2())
+      __m128i vlcn = _mm_set1_epi8('\n');
+      while (s + 15 <= t)
       {
-        __m128i vlcn = _mm_set1_epi8('\n');
-        while (s + 15 <= t)
-        {
-          __m128i vlcm = _mm_loadu_si128(reinterpret_cast<const __m128i*>(s));
-          __m128i vlceq = _mm_cmpeq_epi8(vlcm, vlcn);
-          uint32_t mask = _mm_movemask_epi8(vlceq);
-          n += popcount(mask);
-          s += 16;
-        }
+        __m128i vlcm = _mm_loadu_si128(reinterpret_cast<const __m128i*>(s));
+        __m128i vlceq = _mm_cmpeq_epi8(vlcm, vlcn);
+        uint32_t mask = _mm_movemask_epi8(vlceq);
+        n += popcount(mask);
+        s += 16;
       }
 #elif defined(HAVE_NEON)
       {
@@ -691,7 +685,7 @@ class AbstractMatcher {
 #endif
       uint32_t n0 = 0, n1 = 0, n2 = 0, n3 = 0;
       // clang/gcc 4-way auto-vectorizable loop
-      while (s + 3 <= t)
+      while (s + 3 < t)
       {
         n0 += s[0] == '\n';
         n1 += s[1] == '\n';
@@ -701,23 +695,44 @@ class AbstractMatcher {
       }
       n += n0 + n1 + n2 + n3;
       // epilogue
-      if (s <= t)
+      if (s < t)
       {
         n += *s == '\n';
-        if (++s <= t)
+        if (++s < t)
         {
           n += *s == '\n';
-          if (++s <= t)
+          if (++s < t)
             n += *s == '\n';
         }
       }
-      lno_ = n;
+      // if newlines are detected, then find begin of the last line to adjust bol
+      if (n > 0)
+      {
+        lno_ += n;
+        s = lpb_;
+        // clang/gcc 4-way auto-vectorizable loop
+        while (t - 4 >= s)
+        {
+          if ((t[-1] == '\n') | (t[-2] == '\n') | (t[-3] == '\n') | (t[-4] == '\n'))
+            break;
+          t -= 4;
+        }
+        // epilogue
+        if (--t >= s && *t != '\n')
+          if (--t >= s && *t != '\n')
+            if (--t >= s && *t != '\n')
+              --t;
+        bol_ = t + 1;
+        cpb_ = bol_;
+        cno_ = 0;
+      }
+      lpb_ = txt_;
     }
 #else
     size_t n = lno_;
     size_t k = cno_;
-    char *s = lpb_;
-    char *e = txt_;
+    const char *s = lpb_;
+    const char *e = txt_;
     while (s < e)
     {
       if (*s == '\n')
@@ -765,21 +780,22 @@ class AbstractMatcher {
   {
     (void)lineno();
 #if defined(WITH_SPAN)
-    const char *s = bol_;
+    const char *s = cpb_;
     const char *e = txt_;
-    size_t k = 0;
+    size_t k = cno_;
+    size_t m = opt_.T - 1;
     while (s < e)
     {
       if (*s == '\t')
-        k += 1 + (~k & (opt_.T - 1)); // count tab spacing
+        k += 1 + (~k & m); // count tab spacing
       else
         k += ((*s & 0xC0) != 0x80); // count column offset in UTF-8 chars
       ++s;
     }
-    return k;
-#else
-    return cno_;
+    cpb_ = txt_;
+    cno_ = k;
 #endif
+    return cno_;
   }
   /// Returns the number of columns of the matched text, taking tab spacing into account and counting wide characters as one character each.
   inline size_t columns()
@@ -1057,7 +1073,7 @@ class AbstractMatcher {
     return bol_;
   }
   /// Returns pointer to the end of the line (last char + 1) in the buffer containing the matched text, DANGER: invalidates previous bol() and text() pointers, use eol() before bol(), text(), begin(), and end() when those are used.
-  inline const char *eol(bool inclusive = false) ///< true if inclusive, i.e. point after \n
+  inline const char *eol(bool inclusive = false) ///< true if inclusive, i.e. point after \n instead of at \n
     /// @returns pointer to the end of line
   {
     if (chr_ == '\n' || (txt_ + len_ < buf_ + end_ && txt_[len_] == '\n'))
@@ -1104,7 +1120,7 @@ class AbstractMatcher {
     DBGLOG("AbstractMatcher::span()");
     (void)lineno();
     len_ += txt_ - bol_;
-    txt_ = bol_;
+    txt_ = const_cast<char*>(bol_); // requires ugly cast
     if (chr_ == '\n')
       return txt_;
     reset_text();
@@ -1370,24 +1386,29 @@ class AbstractMatcher {
       return false;
 #if defined(WITH_SPAN)
     (void)lineno();
-    if (bol_ + Const::BLOCK < txt_ && evh_ == NULL)
+    cno_ = 0;
+    if (bol_ + Const::BOLSZ - buf_ < txt_ - bol_ && evh_ == NULL)
     {
-      // this line is very long, likely a binary file, so shift a block size away from the match instead
-      DBGLOG("Line in buffer to long to shift, moving bol position to text match position minus %zu", Const::BLOCK);
-      bol_ = txt_ - Const::BLOCK;
+      // this line is very long, so shift all the way to the match instead of to the begin of the last line
+      DBGLOG("Line in buffer is too long to shift, moving bol position to text match position");
+      (void)columno();
+      bol_ = txt_;
     }
     size_t gap = bol_ - buf_;
-    if (gap > 0 && evh_ != NULL)
-      (*evh_)(*this, buf_, gap, num_);
-    cur_ -= gap;
-    ind_ -= gap;
-    pos_ -= gap;
-    end_ -= gap;
-    txt_ -= gap;
-    bol_ -= gap;
-    lpb_ -= gap;
-    num_ += gap;
-    std::memmove(buf_, buf_ + gap, end_);
+    if (gap > 0)
+    {
+      if (evh_ != NULL)
+        (*evh_)(*this, buf_, gap, num_);
+      cur_ -= gap;
+      ind_ -= gap;
+      pos_ -= gap;
+      end_ -= gap;
+      txt_ -= gap;
+      bol_ -= gap;
+      lpb_ -= gap;
+      num_ += gap;
+      std::memmove(buf_, buf_ + gap, end_);
+    }
     if (max_ - end_ >= need)
     {
       DBGLOG("Shift buffer to close gap of %zu bytes", gap);
@@ -1416,6 +1437,7 @@ class AbstractMatcher {
       buf_ = newbuf;
     }
     bol_ = buf_;
+    cpb_ = buf_;
 #else
     size_t gap = txt_ - buf_;
     if (max_ - end_ + gap >= need)
@@ -1567,32 +1589,33 @@ class AbstractMatcher {
       }
     }
   }
-  Option    opt_; ///< options for matcher engines
-  char     *buf_; ///< input character sequence buffer
-  char     *txt_; ///< points to the matched text in buffer AbstractMatcher::buf_
-  size_t    len_; ///< size of the matched text
-  size_t    cap_; ///< nonzero capture index of an accepted match or zero
-  size_t    cur_; ///< next position in AbstractMatcher::buf_ to assign to AbstractMatcher::txt_
-  size_t    pos_; ///< position in AbstractMatcher::buf_ after AbstractMatcher::txt_
-  size_t    end_; ///< ending position of the input buffered in AbstractMatcher::buf_
-  size_t    max_; ///< total buffer size and max position + 1 to fill
-  size_t    ind_; ///< current indent position
-  size_t    blk_; ///< block size for block-based input reading, as set by AbstractMatcher::buffer
-  int       got_; ///< last unsigned character we looked at (to determine anchors and boundaries)
-  int       chr_; ///< the character located at AbstractMatcher::txt_[AbstractMatcher::len_]
+  Option      opt_; ///< options for matcher engines
+  char       *buf_; ///< input character sequence buffer
+  char       *txt_; ///< points to the matched text in buffer AbstractMatcher::buf_
+  size_t      len_; ///< size of the matched text
+  size_t      cap_; ///< nonzero capture index of an accepted match or zero
+  size_t      cur_; ///< next position in AbstractMatcher::buf_ to assign to AbstractMatcher::txt_
+  size_t      pos_; ///< position in AbstractMatcher::buf_ after AbstractMatcher::txt_
+  size_t      end_; ///< ending position of the input buffered in AbstractMatcher::buf_
+  size_t      max_; ///< total buffer size and max position + 1 to fill
+  size_t      ind_; ///< current indent position
+  size_t      blk_; ///< block size for block-based input reading, as set by AbstractMatcher::buffer
+  int         got_; ///< last unsigned character we looked at (to determine anchors and boundaries)
+  int         chr_; ///< the character located at AbstractMatcher::txt_[AbstractMatcher::len_]
 #if defined(WITH_SPAN)
-  char     *bol_; ///< begin of line pointer in buffer
-  Handler  *evh_; ///< event handler functor to invoke when buffer contents are shifted out
+  const char *bol_; ///< begin of line pointer in buffer
+  Handler    *evh_; ///< event handler functor to invoke when buffer contents are shifted out
 #endif
-  char     *lpb_; ///< line pointer in buffer, updated when counting line numbers with lineno()
-  size_t    lno_; ///< line number count (cached)
-#if !defined(WITH_SPAN)
-  size_t    cno_; ///< column number count (cached)
+  const char *lpb_; ///< line pointer in buffer, updated when counting line numbers with lineno()
+  size_t      lno_; ///< line number count (cached)
+#if defined(WITH_SPAN)
+  const char *cpb_; ///< column pointer in buffer, updated when counting column numbers with columno()
 #endif
-  size_t    num_; ///< character count of the input till bol_
-  bool      own_; ///< true if AbstractMatcher::buf_ was allocated and should be deleted
-  bool      eof_; ///< input has reached EOF
-  bool      mat_; ///< true if AbstractMatcher::matches() was successful
+  size_t      cno_; ///< column number count (cached)
+  size_t      num_; ///< character count of the input till bol_
+  bool        own_; ///< true if AbstractMatcher::buf_ was allocated and should be deleted
+  bool        eof_; ///< input has reached EOF
+  bool        mat_; ///< true if AbstractMatcher::matches() was successful
 };
 
 /// The pattern matcher class template extends abstract matcher base class.
