@@ -299,6 +299,7 @@ class FuzzyMatcher : public Matcher {
     reset_text();
     SaveState sst(ded_);
     len_ = 0; // split text length starts with 0
+    anc_ = false; // no word boundary anchor found and applied
 scan:
     txt_ = buf_ + cur_;
 #if !defined(WITH_NO_INDENT)
@@ -308,7 +309,7 @@ scan:
 #endif
 find:
     int c1 = got_;
-    bool bol = at_bol();
+    bool bol = at_bol(); // at begin of line?
 #if !defined(WITH_NO_INDENT)
 redo:
 #endif
@@ -478,6 +479,7 @@ redo:
                     continue;
                   case Pattern::META_EOL - Pattern::META_MIN:
                     DBGLOG("EOL? %d", c1);
+                    anc_ = true;
                     if (jump == Pattern::Const::IMAX && (c1 == EOF || c1 == '\n' || (c1 == '\r' && peek() == '\n')))
                     {
                       jump = Pattern::index_of(opcode);
@@ -488,6 +490,7 @@ redo:
                     continue;
                   case Pattern::META_BOL - Pattern::META_MIN:
                     DBGLOG("BOL? %d", bol);
+                    anc_ = true;
                     if (jump == Pattern::Const::IMAX && bol)
                     {
                       jump = Pattern::index_of(opcode);
@@ -498,7 +501,8 @@ redo:
                     continue;
                   case Pattern::META_EWE - Pattern::META_MIN:
                     DBGLOG("EWE? %d %d %d", c0, c1, isword(c0) && !isword(c1));
-                    if (jump == Pattern::Const::IMAX && isword(c0) && !isword(c1))
+                    anc_ = true;
+                    if (jump == Pattern::Const::IMAX && (isword(c0) || opt_.W) && !isword(c1))
                     {
                       jump = Pattern::index_of(opcode);
                       if (jump == Pattern::Const::LONG)
@@ -508,6 +512,7 @@ redo:
                     continue;
                   case Pattern::META_BWE - Pattern::META_MIN:
                     DBGLOG("BWE? %d %d %d", c0, c1, !isword(c0) && isword(c1));
+                    anc_ = true;
                     if (jump == Pattern::Const::IMAX && !isword(c0) && isword(c1))
                     {
                       jump = Pattern::index_of(opcode);
@@ -518,6 +523,7 @@ redo:
                     continue;
                   case Pattern::META_EWB - Pattern::META_MIN:
                     DBGLOG("EWB? %d", at_eow());
+                    anc_ = true;
                     if (jump == Pattern::Const::IMAX && isword(got_) &&
                         !isword(static_cast<unsigned char>(method == Const::SPLIT ? txt_[len_] : *txt_)))
                     {
@@ -529,8 +535,9 @@ redo:
                     continue;
                   case Pattern::META_BWB - Pattern::META_MIN:
                     DBGLOG("BWB? %d", at_bow());
+                    anc_ = true;
                     if (jump == Pattern::Const::IMAX && !isword(got_) &&
-                        isword(static_cast<unsigned char>(method == Const::SPLIT ? txt_[len_] : *txt_)))
+                        (opt_.W || isword(static_cast<unsigned char>(method == Const::SPLIT ? txt_[len_] : *txt_))))
                     {
                       jump = Pattern::index_of(opcode);
                       if (jump == Pattern::Const::LONG)
@@ -540,6 +547,7 @@ redo:
                     continue;
                   case Pattern::META_NWE - Pattern::META_MIN:
                     DBGLOG("NWE? %d %d %d", c0, c1, isword(c0) == isword(c1));
+                    anc_ = true;
                     if (jump == Pattern::Const::IMAX && isword(c0) == isword(c1))
                     {
                       jump = Pattern::index_of(opcode);
@@ -550,6 +558,7 @@ redo:
                     continue;
                   case Pattern::META_NWB - Pattern::META_MIN:
                     DBGLOG("NWB? %d %d", at_bow(), at_eow());
+                    anc_ = true;
                     if (jump == Pattern::Const::IMAX &&
                         isword(got_) == isword(static_cast<unsigned char>(txt_[len_])))
                     {
@@ -635,8 +644,8 @@ unrolled:
           jump = Pattern::index_of(opcode);
           if (jump == 0)
           {
-            // loop back to start state: failed to match anything so far?
-            if (cap_ == 0)
+            // loop back to start state after only one char matched (one transition) but w/o full match, then optimize
+            if (cap_ == 0 && pos_ == cur_ + 1 && method == Const::FIND)
               cur_ = pos_; // set cur_ to move forward from cur_ + 1 with FIND advance()
           }
           else if (jump >= Pattern::Const::LONG)
@@ -908,25 +917,54 @@ unrolled:
     }
     if (cap_ == 0)
     {
-      if (method == Const::FIND && !at_end())
+      if (method == Const::FIND)
       {
-        // fuzzy search with find() can safely advance on a single prefix char of the regex
-        if (pos_ > cur_)
+        if (!at_end())
         {
-          // this part is based on advance() in matcher.cpp, limited to advancing ahead till the one of the first pattern char(s) match
-          size_t loc = cur_ + 1;
-          if (pat_->len_ == 0)
+          if (anc_)
           {
-            if (pat_->min_ > 0)
+            cur_ = txt_ - buf_; // reset current to pattern start when a word boundary was encountered
+            anc_ = false;
+          }
+          // fuzzy search with find() can safely advance on a single prefix char of the regex
+          if (pos_ > cur_)
+          {
+            // this part is based on advance() in matcher.cpp, limited to advancing ahead till the one of the first pattern char(s) match
+            size_t loc = cur_ + 1;
+            if (pat_->len_ == 0)
+            {
+              if (pat_->min_ > 0)
+              {
+                const Pattern::Pred *pma = pat_->pma_;
+                while (true)
+                {
+                  const char *s = buf_ + loc;
+                  const char *e = buf_ + end_;
+                  while (s < e && (pma[static_cast<uint8_t>(*s)] & 0xc0) == 0xc0)
+                    ++s;
+                  if (s < e)
+                  {
+                    loc = s - buf_;
+                    set_current(loc);
+                    goto scan;
+                  }
+                  loc = e - buf_;
+                  set_current_match(loc - 1);
+                  peek_more();
+                  loc = cur_ + 1;
+                  if (loc >= end_)
+                    break;
+                }
+              }
+            }
+            else
             {
-              const Pattern::Pred *pma = pat_->pma_;
               while (true)
               {
                 const char *s = buf_ + loc;
                 const char *e = buf_ + end_;
-                while (s < e && (pma[static_cast<uint8_t>(*s)] & 0xc0) == 0xc0)
-                  ++s;
-                if (s < e)
+                s = static_cast<const char*>(std::memchr(s, *pat_->pre_, e - s));
+                if (s != NULL)
                 {
                   loc = s - buf_;
                   set_current(loc);
@@ -936,38 +974,17 @@ unrolled:
                 set_current_match(loc - 1);
                 peek_more();
                 loc = cur_ + 1;
-                if (loc >= end_)
+                if (loc + pat_->len_ > end_)
                   break;
               }
             }
           }
-          else
-          {
-            while (true)
-            {
-              const char *s = buf_ + loc;
-              const char *e = buf_ + end_;
-              s = static_cast<const char*>(std::memchr(s, *pat_->pre_, e - s));
-              if (s != NULL)
-              {
-                loc = s - buf_;
-                set_current(loc);
-                goto scan;
-              }
-              loc = e - buf_;
-              set_current_match(loc - 1);
-              peek_more();
-              loc = cur_ + 1;
-              if (loc + pat_->len_ > end_)
-                break;
-            }
-          }
+          txt_ = buf_ + cur_;
         }
-        txt_ = buf_ + cur_;
       }
       else
       {
-        // no match: backup to begin of unmatched text
+        // SCAN and MATCH: no match: backup to begin of unmatched text to report as error
         cur_ = txt_ - buf_;
       }
     }
