@@ -30,7 +30,7 @@
 @file      flag.hpp
 @brief     Tri-state Flag class and global flags set by options
 @author    Robert van Engelen - engelen@genivia.com
-@copyright (c) 2019-2020, Robert van Engelen, Genivia Inc. All rights reserved.
+@copyright (c) 2019-2022, Robert van Engelen, Genivia Inc. All rights reserved.
 @copyright (c) BSD-3 License - see LICENSE.txt
 */
 
@@ -79,9 +79,11 @@ extern bool flag_cpp;
 extern bool flag_csv;
 extern bool flag_decompress;
 extern bool flag_dereference;
+extern bool flag_files;
 extern bool flag_files_with_matches;
 extern bool flag_files_without_match;
 extern bool flag_fixed_strings;
+extern bool flag_hex_star;
 extern bool flag_hex_cbr;
 extern bool flag_hex_chr;
 extern bool flag_hex_hbr;
@@ -131,6 +133,8 @@ extern Action flag_directories_action;
 extern size_t flag_after_context;
 extern size_t flag_before_context;
 extern size_t flag_fuzzy;
+extern size_t flag_hex_after;
+extern size_t flag_hex_before;
 extern size_t flag_hex_columns;
 extern size_t flag_jobs;
 extern size_t flag_max_count;
@@ -145,6 +149,7 @@ extern size_t flag_min_steal;
 extern size_t flag_not_magic;
 extern size_t flag_query;
 extern size_t flag_tabs;
+extern size_t flag_zmax;
 extern const char *flag_apply_color;
 extern const char *flag_binary_files;
 extern const char *flag_color;
@@ -163,12 +168,13 @@ extern const char *flag_group_separator;
 extern const char *flag_hexdump;
 extern const char *flag_label;
 extern const char *flag_pager;
-extern const char *flag_view;
+extern const char *flag_replace;
 extern const char *flag_save_config;
 extern const char *flag_separator;
 extern const char *flag_sort;
 extern const char *flag_stats;
 extern const char *flag_tag;
+extern const char *flag_view;
 extern std::string flag_config_file;
 extern std::set<std::string> flag_config_options;
 extern std::vector<std::string> flag_regexp;
