@@ -30,7 +30,7 @@
 @file      output.cpp
 @brief     Output management
 @author    Robert van Engelen - engelen@genivia.com
-@copyright (c) 2019-2020, Robert van Engelen, Genivia Inc. All rights reserved.
+@copyright (c) 2019-2022, Robert van Engelen, Genivia Inc. All rights reserved.
 @copyright (c) BSD-3 License - see LICENSE.txt
 */
 
@@ -52,6 +52,33 @@ void Output::Dump::hex(short mode, size_t byte_offset, const char *data, size_t
 // dump one line of hex
 void Output::Dump::line()
 {
+  if (flag_hex_star)
+  {
+    // if the previous hex line was the same as this hex line, output a * (but only once)
+    size_t i;
+
+    for (i = 0; i < flag_hex_columns; ++i)
+      if (prevb[i] < 0 || bytes[i] != prevb[i])
+        break;
+
+    if (i >= flag_hex_columns)
+    {
+      if (!pstar)
+      {
+        out.str(color_se);
+        out.chr('*');
+        out.nl();
+
+        pstar = true;
+      }
+
+      for (i = 0; i < MAX_HEX_COLUMNS; ++i)
+        bytes[i] = -1;
+
+      return;
+    }
+  }
+
   out.str(color_bn);
   out.hex((offset - 1) - (offset - 1) % flag_hex_columns, 8);
   out.str(color_off);
@@ -175,8 +202,13 @@ void Output::Dump::line()
 
   out.nl();
 
-  for (size_t i = 0; i < MAX_HEX_COLUMNS; ++i)
+  for (int i = 0; i < MAX_HEX_COLUMNS; ++i)
+  {
+    prevb[i] = bytes[i];
     bytes[i] = -1;
+  }
+
+  pstar = false;
 }
 
 // output the header part of the match, preceding the matched line
@@ -195,7 +227,7 @@ void Output::header(const char *& pathname, const std::string& partname, size_t
   bool nul = false; // -Q: mark pathname with three NUL bytes unless -a
 
   if (flag_with_filename && pathname != NULL)
-  { 
+  {
     nul = flag_query > 0 && !flag_text;
 
     if (nul)
@@ -233,7 +265,7 @@ void Output::header(const char *& pathname, const std::string& partname, size_t
       str(color_fn);
       str(color_del);
       str(color_off);
-      chr('\n');
+      nl();
       pathname = NULL;
     }
     else
@@ -245,7 +277,7 @@ void Output::header(const char *& pathname, const std::string& partname, size_t
   if (!flag_no_filename && !partname.empty())
   {
     nul = flag_query > 0 && !flag_text && (flag_heading || !nul);
-    
+
     if (nul)
       chr('\0');
 
@@ -331,6 +363,8 @@ void Output::header(const char *& pathname, const std::string& partname, size_t
 // output "Binary file ... matches"
 void Output::binary_file_matches(const char *pathname, const std::string& partname)
 {
+  if ((mode_ & BINARY) != 0)
+    return;
   str(color_off);
   str("Binary file");
   str(color_fn);
@@ -362,6 +396,60 @@ void Output::binary_file_matches(const char *pathname, const std::string& partna
   str(color_off);
   str(" matches");
   nl();
+  mode_ |= BINARY;
+}
+
+// get a group capture's string pointer and size specified by %[ARG] as arg, if any
+std::pair<const char*,size_t> Output::capture(reflex::AbstractMatcher *matcher, const char *arg)
+{
+  if (arg != NULL)
+  {
+    while (true)
+    {
+      const char *bar = strchr(arg, '|');
+      const char *end = strchr(arg, ']');
+
+      if (end == NULL)
+        break;
+
+      if (bar == NULL || bar > end)
+        bar = end;
+
+      if (isdigit(*arg))
+      {
+        size_t index = strtoul(arg, NULL, 10);
+
+        if (index == 0 || *bar == ']')
+          return (*matcher)[index];
+
+        std::pair<size_t,const char*> id = matcher->group_id();
+
+        while (id.first != 0 && id.first != index)
+          id = matcher->group_next_id();
+
+        if (id.first == index)
+          return (*matcher)[index];
+      }
+      else
+      {
+        std::pair<size_t,const char*> id = matcher->group_id();
+
+        while (id.first != 0
+            && (id.second == NULL || strncmp(id.second, arg, bar - arg) != 0 || id.second[bar - arg] != '\0'))
+          id = matcher->group_next_id();
+
+        if (id.first != 0)
+          return (*matcher)[id.first];
+      }
+
+      if (*bar == ']')
+        break;
+
+      arg = bar + 1;
+    }
+  }
+
+  return std::pair<const char*,size_t>(NULL, 0);
 }
 
 // output formatted match with options --format, --format-open, --format-close
@@ -378,7 +466,7 @@ void Output::format(const char *format, const char *pathname, const std::string&
 
   while (*s != '\0')
   {
-    const char *a = NULL;
+    const char *arg = NULL;
     const char *t = s;
 
     while (*s != '\0' && *s != '%')
@@ -389,7 +477,7 @@ void Output::format(const char *format, const char *pathname, const std::string&
     ++s;
     if (*s == '[')
     {
-      a = ++s;
+      arg = ++s;
       while (*s != '\0' && *s != ']')
         ++s;
       if (*s == '\0' || *(s + 1) == '\0')
@@ -404,8 +492,8 @@ void Output::format(const char *format, const char *pathname, const std::string&
       case 'F':
         if (flag_with_filename && pathname != NULL)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           str(pathname);
           if (!partname.empty())
           {
@@ -460,8 +548,8 @@ void Output::format(const char *format, const char *pathname, const std::string&
       case 'H':
         if (flag_with_filename)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           if (!partname.empty())
           {
             std::string name(pathname);
@@ -499,8 +587,8 @@ void Output::format(const char *format, const char *pathname, const std::string&
       case 'N':
         if (flag_line_number)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           num(matcher->lineno());
           if (sep != NULL)
             str(sep, len);
@@ -516,8 +604,8 @@ void Output::format(const char *format, const char *pathname, const std::string&
       case 'K':
         if (flag_column_number)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           num(matcher->columno() + 1);
           if (sep != NULL)
             str(sep, len);
@@ -533,8 +621,8 @@ void Output::format(const char *format, const char *pathname, const std::string&
       case 'B':
         if (flag_byte_offset)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           num(matcher->first());
           if (sep != NULL)
             str(sep, len);
@@ -544,14 +632,23 @@ void Output::format(const char *format, const char *pathname, const std::string&
         break;
 
       case 'b':
-        num(matcher->first());
+        if (arg != NULL)
+        {
+          std::pair<const char*,size_t> cap = capture(matcher, arg);
+          if (cap.first != NULL)
+            num(cap.first - matcher->text() + matcher->first());
+        }
+        else
+        {
+          num(matcher->first());
+        }
         break;
 
       case 'T':
         if (flag_initial_tab)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           chr('\t');
         }
         break;
@@ -563,8 +660,8 @@ void Output::format(const char *format, const char *pathname, const std::string&
       case 'S':
         if (next)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           if (sep != NULL)
             str(sep, len);
           else
@@ -584,11 +681,29 @@ void Output::format(const char *format, const char *pathname, const std::string&
         break;
 
       case 'd':
-        num(matcher->size());
+        if (arg != NULL)
+        {
+          std::pair<const char*,size_t> cap = capture(matcher, arg);
+          if (cap.first != NULL)
+            num(cap.second);
+        }
+        else
+        {
+          num(matcher->size());
+        }
         break;
 
       case 'e':
-        num(matcher->last());
+        if (arg != NULL)
+        {
+          std::pair<const char*,size_t> cap = capture(matcher, arg);
+          if (cap.first != NULL)
+            num(cap.first + cap.second - matcher->text() + matcher->first());
+        }
+        else
+        {
+          num(matcher->last());
+        }
         break;
 
       case 'G':
@@ -608,26 +723,33 @@ void Output::format(const char *format, const char *pathname, const std::string&
 
           colon = true;
 
-          if (a != NULL)
+          if (arg != NULL)
           {
             size_t n = id.first;
-            const char *b;
+            const char *bar;
+            const char *end;
 
             while (true)
             {
-              b = strchr(a, '|');
+              bar = strchr(arg, '|');
+              end = strchr(arg, ']');
 
-              if (--n == 0 || b == NULL)
+              --n;
+
+              if (bar == NULL || (end != NULL && bar > end))
+              {
+                bar = end;
+                break;
+              }
+
+              if (n == 0)
                 break;
 
-              a = b + 1;
+              arg = bar + 1;
             }
 
-            if (b == NULL)
-              b = strchr(a, ']');
-
-            if (n == 0 && b != NULL)
-              str(a, b - a);
+            if (n == 0 && bar != NULL)
+              str(arg, bar - arg);
             else if (id.second != NULL)
               str(id.second);
             else
@@ -647,30 +769,37 @@ void Output::format(const char *format, const char *pathname, const std::string&
       }
 
       case 'g':
-        if (a != NULL)
+        if (arg != NULL)
         {
           std::pair<size_t,const char*> id = matcher->group_id();
           size_t n = id.first;
 
           if (n > 0)
           {
-            const char *b;
+            const char *bar;
+            const char *end;
 
             while (true)
             {
-              b = strchr(a, '|');
+              bar = strchr(arg, '|');
+              end = strchr(arg, ']');
 
-              if (--n == 0 || b == NULL)
+              --n;
+
+              if (bar == NULL || (end != NULL && bar > end))
+              {
+                bar = end;
+                break;
+              }
+
+              if (n == 0)
                 break;
 
-              a = b + 1;
+              arg = bar + 1;
             }
 
-            if (b == NULL)
-              b = strchr(a, ']');
-
-            if (n == 0 && b != NULL)
-              str(a, b - a);
+            if (n == 0 && bar != NULL)
+              str(arg, bar - arg);
             else if (id.second != NULL)
               str(id.second);
             else
@@ -778,6 +907,7 @@ void Output::format(const char *format, const char *pathname, const std::string&
       case 'Z':
         if (flag_fuzzy > 0)
         {
+          // --Z: we used the fuzzy matcher to search, so a dynamic cast is fine
           reflex::FuzzyMatcher *fuzzy_matcher = dynamic_cast<reflex::FuzzyMatcher*>(matcher);
           num(fuzzy_matcher->edits());
         }
@@ -789,22 +919,25 @@ void Output::format(const char *format, const char *pathname, const std::string&
         break;
 
       case '$':
-        sep = a;
-        len = s - a - 1;
+        sep = arg;
+        len = s - arg - 1;
         break;
 
       case '~':
+#ifdef OS_WIN
+        chr('\r');
+#endif
         chr('\n');
         break;
 
       case '<':
-        if (!next && a != NULL)
-          str(a, s - a - 1);
+        if (!next && arg != NULL)
+          str(arg, s - arg - 1);
         break;
 
       case '>':
-        if (next && a != NULL)
-          str(a, s - a - 1);
+        if (next && arg != NULL)
+          str(arg, s - arg - 1);
         break;
 
       case ',':
@@ -815,6 +948,14 @@ void Output::format(const char *format, const char *pathname, const std::string&
           chr(c);
         break;
 
+      case '#':
+      {
+        std::pair<const char*,size_t> cap = capture(matcher, arg);
+        if (cap.first != NULL)
+          str(cap.first, cap.second);
+        break;
+      }
+
       default:
         chr(c);
         break;
@@ -829,76 +970,8 @@ void Output::format(const char *format, const char *pathname, const std::string&
       case '7':
       case '8':
       case '9':
-      case '#':
-        if (c == '#')
-        {
-          if (a != NULL)
-          {
-            while (true)
-            {
-              const char *b = strchr(a, '|');
-
-              if (b == NULL)
-              {
-                b = strchr(a, ']');
-
-                if (b == NULL)
-                  break;
-              }
-
-              if (isdigit(*a))
-              {
-                size_t index = strtoul(a, NULL, 10);
-
-                if (index == 0 || *b == ']')
-                {
-                  std::pair<const char*,size_t> capture = (*matcher)[index];
-                  str(capture.first, capture.second);
-                  break;
-                }
-                else
-                {
-                  std::pair<size_t,const char*> id = matcher->group_id();
-
-                  while (id.first != 0 && id.first != index)
-                    id = matcher->group_next_id();
-
-                  if (id.first == index)
-                  {
-                    std::pair<const char*,size_t> capture = (*matcher)[index];
-                    str(capture.first, capture.second);
-                    break;
-                  }
-                }
-              }
-              else
-              {
-                std::pair<size_t,const char*> id = matcher->group_id();
-
-                while (id.first != 0
-                    && (id.second == NULL || strncmp(id.second, a, b - a) != 0 || id.second[b - a] != '\0'))
-                  id = matcher->group_next_id();
-
-                if (id.first != 0)
-                {
-                  std::pair<const char*,size_t> capture = (*matcher)[id.first];
-                  str(capture.first, capture.second);
-                  break;
-                }
-              }
-
-              if (*b == ']')
-                break;
-
-              a = b + 1;
-            }
-          }
-        }
-        else
-        {
-          std::pair<const char*,size_t> capture = (*matcher)[c - '0'];
-          str(capture.first, capture.second);
-        }
+        std::pair<const char*,size_t> capture = (*matcher)[c - '0'];
+        str(capture.first, capture.second);
         break;
     }
     ++s;
@@ -914,7 +987,7 @@ void Output::format_invert(const char *format, const char *pathname, const std::
 
   while (*s != '\0')
   {
-    const char *a = NULL;
+    const char *arg = NULL;
     const char *t = s;
 
     while (*s != '\0' && *s != '%')
@@ -925,7 +998,7 @@ void Output::format_invert(const char *format, const char *pathname, const std::
     ++s;
     if (*s == '[')
     {
-      a = ++s;
+      arg = ++s;
       while (*s != '\0' && *s != ']')
         ++s;
       if (*s == '\0' || *(s + 1) == '\0')
@@ -940,8 +1013,8 @@ void Output::format_invert(const char *format, const char *pathname, const std::
       case 'F':
         if (flag_with_filename && pathname != NULL)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           str(pathname);
           if (!partname.empty())
           {
@@ -996,8 +1069,8 @@ void Output::format_invert(const char *format, const char *pathname, const std::
       case 'H':
         if (flag_with_filename)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           if (!partname.empty())
           {
             std::string name(pathname);
@@ -1035,8 +1108,8 @@ void Output::format_invert(const char *format, const char *pathname, const std::
       case 'N':
         if (flag_line_number)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           num(lineno);
           if (sep != NULL)
             str(sep, len);
@@ -1052,8 +1125,8 @@ void Output::format_invert(const char *format, const char *pathname, const std::
       case 'K':
         if (flag_column_number)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           chr('1');
           if (sep != NULL)
             str(sep, len);
@@ -1069,8 +1142,8 @@ void Output::format_invert(const char *format, const char *pathname, const std::
       case 'B':
         if (flag_byte_offset)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           num(offset);
           if (sep != NULL)
             str(sep, len);
@@ -1086,8 +1159,8 @@ void Output::format_invert(const char *format, const char *pathname, const std::
       case 'T':
         if (flag_initial_tab)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           chr('\t');
         }
         break;
@@ -1099,8 +1172,8 @@ void Output::format_invert(const char *format, const char *pathname, const std::
       case 'S':
         if (next)
         {
-          if (a != NULL)
-            str(a, s - a - 1);
+          if (arg != NULL)
+            str(arg, s - arg - 1);
           if (sep != NULL)
             str(sep, len);
           else
@@ -1118,8 +1191,8 @@ void Output::format_invert(const char *format, const char *pathname, const std::
       case 'w':
       {
         size_t n = 0;
-        for (const char *s = ptr; s < ptr + size; ++s)
-          n += (*s & 0xC0) != 0x80;
+        for (const char *p = ptr; p < ptr + size; ++p)
+          n += (*p & 0xC0) != 0x80;
         num(n);
         break;
       }
@@ -1197,22 +1270,25 @@ void Output::format_invert(const char *format, const char *pathname, const std::
         break;
 
       case '$':
-        sep = a;
-        len = s - a - 1;
+        sep = arg;
+        len = s - arg - 1;
         break;
 
       case '~':
+#ifdef OS_WIN
+        chr('\r');
+#endif
         chr('\n');
         break;
 
       case '<':
-        if (!next && a != NULL)
-          str(a, s - a - 1);
+        if (!next && arg != NULL)
+          str(arg, s - arg - 1);
         break;
 
       case '>':
-        if (next && a != NULL)
-          str(a, s - a - 1);
+        if (next && arg != NULL)
+          str(arg, s - arg - 1);
         break;
 
       case ',':
@@ -1223,10 +1299,7 @@ void Output::format_invert(const char *format, const char *pathname, const std::
           chr(c);
         break;
 
-      default:
-        chr(c);
-        break;
-
+      case '#':
       case '0':
       case '1':
       case '2':
@@ -1237,12 +1310,16 @@ void Output::format_invert(const char *format, const char *pathname, const std::
       case '7':
       case '8':
       case '9':
-      case '#':
+        break;
+
+      default:
+        chr(c);
         break;
     }
     ++s;
   }
 }
+
 // output a quoted string with escapes for \ and "
 void Output::quote(const char *data, size_t size)
 {
