@@ -30,7 +30,7 @@
 @file      query.cpp
 @brief     Query engine and UI
 @author    Robert van Engelen - engelen@genivia.com
-@copyright (c) 2019-2020, Robert van Engelen, Genivia Inc. All rights reserved.
+@copyright (c) 2019-2022, Robert van Engelen, Genivia Inc. All rights reserved.
 @copyright (c) BSD-3 License - see LICENSE.txt
 */
 
@@ -650,7 +650,7 @@ void Query::query()
   get_stdin();
 
   if (!VKey::setup(VKey::TTYRAW))
-    abort("no keyboard detected");
+    abort("no ANSI terminal keyboard detected");
 
   if (!Screen::setup("ugrep --query"))
   {
@@ -1338,11 +1338,12 @@ void Query::search()
 
   if (search_thread_.joinable())
   {
+    // clear "Searching..." or "(END)", if displayed
     if (error_ == -1 && rows_ < row_ + Screen::rows - 1)
     {
       Screen::normal();
       Screen::invert();
-      Screen::put(rows_ - row_ + 1, 0, "WAIT");
+      Screen::put(rows_ - row_ + 1, 0, "Restarting search...");
       Screen::normal();
       Screen::erase();
     }
@@ -1647,7 +1648,7 @@ void Query::fetch_all()
 // execute the search in a new thread
 void Query::execute(int fd)
 {
-  output = fdopen(fd, "w");
+  output = fdopen(fd, "wb");
 
   if (output != NULL)
   {
@@ -2925,24 +2926,25 @@ void Query::meta(int key)
         {
           flags_[30].flag = true;
 
-          if (fuzzy_ > 1)
+          if ((fuzzy_ & 0xff) > 1)
             --fuzzy_;
 
-          msg.append(" to ").append(std::to_string(fuzzy_));
+          msg.append(" to ").append(std::to_string(fuzzy_ & 0xff));
         }
         else if (key == ']')
         {
-          if (flags_[30].flag)
+          if (flags_[30].flag && (fuzzy_ & 0xff) < 0xff)
             ++fuzzy_;
           else
             flags_[30].flag = true;
 
-          msg.append(" to ").append(std::to_string(fuzzy_));
+          msg.append(" to ").append(std::to_string(fuzzy_ & 0xff));
         }
         else if (key == '{')
         {
-          if (!flags_[0].flag && !flags_[1].flag)
-            flags_[3].flag = true;
+          if (!flags_[26].flag || flag_hexdump == NULL)
+            if (!flags_[0].flag && !flags_[1].flag)
+              flags_[3].flag = true;
 
           if (context_ > 1)
             --context_;
@@ -2951,10 +2953,12 @@ void Query::meta(int key)
         }
         else if (key == '}')
         {
-          if (flags_[0].flag || flags_[1].flag || flags_[3].flag)
+          if (flags_[26].flag && flag_hexdump != NULL)
+            ++context_;
+          else if (flags_[0].flag || flags_[1].flag || flags_[3].flag)
             ++context_;
           else if (!flags_[0].flag && !flags_[1].flag)
-             flags_[3].flag = true;
+            flags_[3].flag = true;
 
           msg.append(" to ").append(std::to_string(context_));
         }
@@ -3201,6 +3205,16 @@ void Query::get_flags()
   else if (flag_before_context > 0)
     context_ = flag_before_context;
 
+  if (flag_hexdump != NULL)
+  {
+    if (flag_hex_after > 0)
+      context_ = flag_hex_after;
+    else if (flag_hex_before > 0)
+      context_ = flag_hex_before;
+    else
+      context_ = 0;
+  }
+
   // remember the --fuzzy max, when specified
   if (flag_fuzzy > 0)
     fuzzy_ = flag_fuzzy;
@@ -3291,8 +3305,21 @@ void Query::set_flags()
   flag_no_messages = true;
 
   // set ugrep flags to the interactive flags
-  flag_after_context = context_ * (flags_[0].flag || flags_[3].flag);
-  flag_before_context = context_ * (flags_[1].flag || flags_[3].flag);
+  if (flags_[26].flag && flag_hexdump != NULL)
+  {
+    flag_hex_after = flag_hex_before = context_;
+    flag_after_context = flag_before_context = 0;
+  }
+  else
+  {
+    flag_after_context = context_ * (flags_[0].flag || flags_[3].flag);
+    flag_before_context = context_ * (flags_[1].flag || flags_[3].flag);
+    if (flag_hexdump != NULL)
+    {
+      flag_hex_after = (flag_after_context == 0);
+      flag_hex_before = (flag_before_context == 0);
+    }
+  }
   flag_byte_offset = flags_[2].flag;
   flag_count = flags_[4].flag;
   flag_fixed_strings = flags_[5].flag;
@@ -3432,7 +3459,7 @@ void Query::set_stdin()
       return;
     }
 
-    source = fdopen(stdin_pipe_[0], "r");
+    source = fdopen(stdin_pipe_[0], "rb");
 
     stdin_thread_ = std::thread(Query::stdin_sender, stdin_pipe_[1]);
   }
