@@ -30,7 +30,7 @@
 @file      stats.cpp
 @brief     collect global statistics - static, partially thread-safe
 @author    Robert van Engelen - engelen@genivia.com
-@copyright (c) 2019-2020, Robert van Engelen, Genivia Inc. All rights reserved.
+@copyright (c) 2019-2022, Robert van Engelen, Genivia Inc. All rights reserved.
 @copyright (c) BSD-3 License - see LICENSE.txt
 */
 
@@ -54,55 +54,57 @@ void Stats::report(FILE *output)
     fprintf(output, " in %zu director%s", sd, (sd == 1 ? "y" : "ies"));
   fprintf(output, ": %zu matching", ff);
   if (fp > ff)
-    fprintf(output, " + %zu in archives\n", fp - ff);
+    fprintf(output, " + %zu in archives" NEWLINESTR, fp - ff);
   else
-    fprintf(output, "\n");
+    fprintf(output, NEWLINESTR);
   if (warnings > 0)
-    fprintf(output, "Received %zu warning%s\n", ws, ws == 1 ? "" : "s");
+    fprintf(output, "Received %zu warning%s" NEWLINESTR, ws, ws == 1 ? "" : "s");
 
-  fprintf(output, "The following pathname selections and restrictions were applied:\n");
+  fprintf(output, "The following pathname selections and restrictions were applied:" NEWLINESTR);
   if (flag_config != NULL)
-    fprintf(output, "  --config=%s\n", flag_config_file.c_str());
+    fprintf(output, "  --config=%s" NEWLINESTR, flag_config_file.c_str());
+  if (flag_min_depth > 0 && flag_max_depth > 0)
+    fprintf(output, "  --depth=%zu,%zu" NEWLINESTR, flag_min_depth, flag_max_depth);
+  else if (flag_min_depth > 0)
+    fprintf(output, "  --depth=%zu," NEWLINESTR, flag_min_depth);
+  else if (flag_max_depth > 0)
+    fprintf(output, "  --depth=%zu" NEWLINESTR, flag_max_depth);
+  if (flag_files)
+    fprintf(output, "  --files" NEWLINESTR);
 #ifdef WITH_HIDDEN
   if (flag_hidden)
-    fprintf(output, "  --hidden (default)\n");
+    fprintf(output, "  --hidden (default)" NEWLINESTR);
   else
-    fprintf(output, "  --no-hidden\n");
+    fprintf(output, "  --no-hidden" NEWLINESTR);
 #else
   if (flag_hidden)
-    fprintf(output, "  --hidden\n");
+    fprintf(output, "  --hidden" NEWLINESTR);
   else
-    fprintf(output, "  --no-hidden (default)\n");
+    fprintf(output, "  --no-hidden (default)" NEWLINESTR);
 #endif
-  if (flag_min_depth > 0 && flag_max_depth > 0)
-    fprintf(output, "  --depth=%zu,%zu\n", flag_min_depth, flag_max_depth);
-  else if (flag_min_depth > 0)
-    fprintf(output, "  --depth=%zu,\n", flag_min_depth);
-  else if (flag_max_depth > 0)
-    fprintf(output, "  --depth=%zu\n", flag_max_depth);
   for (auto& i : flag_ignore_files)
-    fprintf(output, "  --ignore-files='%s'\n", i.c_str());
+    fprintf(output, "  --ignore-files='%s'" NEWLINESTR, i.c_str());
   for (auto& i : ignore)
-    fprintf(output, "    %s exclusions were applied to %s\n", i.c_str(), i.substr(0, i.find_last_of(PATHSEPCHR)).c_str());
+    fprintf(output, "    %s exclusions were applied to %s and its subdirectories" NEWLINESTR, i.c_str(), i.substr(0, i.find_last_of(PATHSEPCHR)).c_str());
   for (auto& i : flag_file_magic)
   {
     if (!i.empty() && (i.front() == '!' || i.front() == '^'))
-      fprintf(output, "  --file-magic='!%s' (negation)\n", i.c_str() + 1);
+      fprintf(output, "  --file-magic='!%s' (negation)" NEWLINESTR, i.c_str() + 1);
     else
-      fprintf(output, "  --file-magic='%s'\n", i.c_str());
+      fprintf(output, "  --file-magic='%s'" NEWLINESTR, i.c_str());
   }
   for (auto& i : flag_include_fs)
-    fprintf(output, "  --include-fs='%s'\n", i.c_str());
+    fprintf(output, "  --include-fs='%s'" NEWLINESTR, i.c_str());
   for (auto& i : flag_exclude_fs)
-    fprintf(output, "  --exclude-fs='%s'\n", i.c_str());
+    fprintf(output, "  --exclude-fs='%s'" NEWLINESTR, i.c_str());
   for (auto& i : flag_all_include)
-    fprintf(output, "  --include='%s'%s\n", i.c_str(), i.front() == '!' ? " (negated)" : "");
+    fprintf(output, "  --include='%s'%s" NEWLINESTR, i.c_str(), i.front() == '!' ? " (negated)" : "");
   for (auto& i : flag_all_exclude)
-    fprintf(output, "  --exclude='%s'%s\n", i.c_str(), i.front() == '!' ? " (negated)" : "");
+    fprintf(output, "  --exclude='%s'%s" NEWLINESTR, i.c_str(), i.front() == '!' ? " (negated)" : "");
   for (auto& i : flag_all_include_dir)
-    fprintf(output, "  --include-dir='%s'%s\n", i.c_str(), i.front() == '!' ? " (negated)" : "");
+    fprintf(output, "  --include-dir='%s'%s" NEWLINESTR, i.c_str(), i.front() == '!' ? " (negated)" : "");
   for (auto& i : flag_all_exclude_dir)
-    fprintf(output, "  --exclude-dir='%s'%s\n", i.c_str(), i.front() == '!' ? " (negated)" : "");
+    fprintf(output, "  --exclude-dir='%s'%s" NEWLINESTR, i.c_str(), i.front() == '!' ? " (negated)" : "");
 }
 
 size_t                   Stats::files  = 0;
