@@ -1,25 +1,28 @@
 #!/bin/bash
 
-UG="../src/ugrep --color=always --sort $@"
+UGREP=${UGREP_ABS_PATH:-../src/ugrep}
+CONFIGH=${CONFIGH_ABS_PATH:-../config.h}
 
-FILES="Hello.bat Hello.class Hello.java Hello.pdf Hello.sh Hello.txt"
+UG="$UGREP --color=always --sort $@"
 
-if test ! -x "../src/ugrep" ; then
-  echo "../src/ugrep not found, exiting"
+FILES="Hello.bat Hello.class Hello.java Hello.pdf Hello.sh Hello.txt empty.txt emptyline.txt"
+
+if test ! -x "$UGREP" ; then
+  echo "$UGREP not found, exiting"
   exit 1
 fi
 
-if test ! -e ../config.h ; then
-  echo "../config.h not found, exiting"
+if test ! -e "$CONFIGH" ; then
+  echo "$CONFIGH not found, exiting"
   exit 1
 fi
 
 if ! $UG --version | head -n1 ; then
-  echo "../src/ugrep failed to execute, exiting"
+  echo "$UGREP failed to execute, exiting"
   exit 1
 fi
 
-if $UG -Fq 'HAVE_PCRE2 1' ../config.h ; then
+if $UG -Fq 'HAVE_PCRE2 1' "$CONFIGH" ; then
   have_pcre2=yes
 else
   have_pcre2=no
@@ -28,7 +31,7 @@ fi
 echo "Have libpcre2?" $have_pcre2
 
 if test "$have_pcre2" = "no" ; then
-  if $UG -Fq 'HAVE_BOOST_REGEX 1' ../config.h ; then
+  if $UG -Fq 'HAVE_BOOST_REGEX 1' "$CONFIGH" ; then
     have_boost_regex=yes
   else
     have_boost_regex=no
@@ -37,7 +40,7 @@ if test "$have_pcre2" = "no" ; then
   echo "Have libboost_regex?" $have_boost_regex
 fi
 
-if $UG -Fq 'HAVE_LIBZ 1' ../config.h ; then
+if $UG -Fq 'HAVE_LIBZ 1' "$CONFIGH" ; then
   have_libz=yes
 else
   have_libz=no
@@ -45,7 +48,7 @@ fi
 
 echo "Have libz?" $have_libz
 
-if $UG -Fq 'HAVE_LIBBZ2 1' ../config.h ; then
+if $UG -Fq 'HAVE_LIBBZ2 1' "$CONFIGH" ; then
   have_libbz2=yes
 else
   have_libbz2=no
@@ -53,7 +56,7 @@ fi
 
 echo "Have libbz2?" $have_libbz2
 
-if $UG -Fq 'HAVE_LIBLZMA 1' ../config.h ; then
+if $UG -Fq 'HAVE_LIBLZMA 1' "$CONFIGH" ; then
   have_liblzma=yes
 else
   have_liblzma=no
@@ -61,7 +64,7 @@ fi
 
 echo "Have liblzma?" $have_liblzma
 
-if $UG -Fq 'HAVE_LIBLZ4 1' ../config.h ; then
+if $UG -Fq 'HAVE_LIBLZ4 1' "$CONFIGH" ; then
   have_liblz4=yes
 else
   have_liblz4=no
@@ -69,7 +72,7 @@ fi
 
 echo "Have liblz4?" $have_liblz4
 
-if $UG -Fq 'HAVE_LIBZSTD 1' ../config.h ; then
+if $UG -Fq 'HAVE_LIBZSTD 1' "$CONFIGH" ; then
   have_libzstd=yes
 else
   have_libzstd=no
@@ -93,6 +96,7 @@ mkdir -p dir1 dir2
 ln -s ../Hello.java dir1
 cp Hello.sh dir1
 cp Hello.bat dir1
+cp makefile dir1
 ln -s ../Hello.java dir2
 cp Hello.sh dir2
 cp Hello.bat dir2
@@ -114,7 +118,7 @@ $UG -Rl -Osh                             Hello dir1 | $DIFF out/dir-O.out
 printf .
 $UG -Rl -M'#!/bin/sh'                    Hello dir1 | $DIFF out/dir-M.out             || ERR "-Rl -M'#!/bin/sh' Hello dir1"
 printf .
-$UG -Rl -tShell                          Hello dir1 | $DIFF out/dir-t.out             || ERR "-Rl -tShell Hello dir1"
+$UG -Rl -tShell,make                     Hello dir1 | $DIFF out/dir-t.out             || ERR "-Rl -tShell,make Hello dir1"
 printf .
 $UG -1l                                  Hello dir1 | $DIFF out/dir-1.out             || ERR "-1l Hello dir1"
 printf .
@@ -242,6 +246,10 @@ for PAT in '' 'Hello' '\w+\s+\S+' '\S\n\S' 'nomatch' ; do
     | $DIFF "out/$FN-v--format.out" \
     || ERR "-v --format-open='%m) %f:%~' --format='  %m) %n,%k %w-%d%~' --format-close='%~' '$PAT' $FILES"
   printf .
+  $UG -U --replace='(%m:%o)' "$PAT" $FILES \
+    | $DIFF "out/$FN--replace.out" \
+    || ERR "--replace='(%m:%o)' '$PAT' $FILES"
+  printf .
   $UG -U -Iw "$PAT" $FILES \
     | $DIFF "out/$FN-Iw.out" \
     || ERR "-Iw '$PAT' $FILES"
@@ -299,12 +307,17 @@ for PAT in '' 'Hello' '\w+\s+\S+' '\S\n\S' 'nomatch' ; do
   fi
 done
 
-for PAT in '' 'Hello World' 'Hello -World' 'Hello -World|greeting' 'Hello -(greeting|World)' '"a Hello" greeting' ; do
+for PAT in '' 'Hello World' 'Hello -World' 'Hello -bin' 'bin -Hello' 'bin -greeting' 'Hello -World|greeting' 'Hello -bin|greeting' 'Hello -(greeting|World)' '"a Hello" greeting' ; do
   FN=`echo "Hello_$PAT" | tr -Cd '[:alnum:]_-'`
-  printf .
-  $UG -U --bool "$PAT" $FILES \
-    | $DIFF "out/$FN--bool.out" \
-    || ERR "--bool '$PAT' $FILES"
+  for OPS in '' '-l' '-c' '-co' '-o' '-C2' '-y' '--json' ; do
+    printf .
+    $UG -U --bool $OPS "$PAT" $FILES \
+      | $DIFF "out/$FN--bool$OPS.out" \
+      || ERR "--bool $OPS '$PAT' $FILES"
+    $UG -U --files --bool $OPS "$PAT" $FILES \
+      | $DIFF "out/$FN--files--bool$OPS.out" \
+      || ERR "--files --bool $OPS '$PAT' $FILES"
+  done
 done
 
 printf .
@@ -390,6 +403,18 @@ $UG -z -c -tShell Hello archive.tzst    | $DIFF out/archive-t.tzst.out    || ERR
 fi
 fi
 
+# --zmax has shown to be stable unless we can find a counter example
+#if [ "$have_libz" == yes ]; then
+#printf .
+#$UG --zmax=2 -z -c Hello archive2.tgz         | $DIFF out/archive2.tgz.out   || ERR "-zmax=2 -z -c Hello archive2.tgz"
+#printf .
+#$UG --zmax=3 -z -c Hello archive3.tgz         | $DIFF out/archive3.tgz.out   || ERR "-zmax=3 -z -c Hello archive3.tgz"
+#printf .
+#$UG --zmax=2 -z -c -tShell Hello archive2.tgz | $DIFF out/archive2-t.tgz.out || ERR "-zmax=2 -z -c -tShell Hello archive2.tgz"
+#printf .
+#$UG --zmax=3 -z -c -tShell Hello archive3.tgz | $DIFF out/archive3-t.tgz.out || ERR "-zmax=3 -z -c -tShell Hello archive3.tgz"
+#fi
+
 if [ "$have_libz" == yes ]; then
 printf .
 $UG -z -c '' archive.gz | $DIFF out/archive.gz.out || ERR "-z -c '' archive.gz"
